VERSION = '0.0.57'
ROOT_DIR = '/root'
HOST_ADDRESS = 'eubflow.eubchain.com'
HTTP_ADDRESS = 'https://%s' % HOST_ADDRESS
WS_ADDRESS = 'wss://%s:54188/' % HOST_ADDRESS
ENABLE_VERBOSE = False
DOCKER_WHITE_LIST = ['rancher_agent']